/* AD9851-EWD
 * DDS Signal Generator
 * with AD9851
 * 
 * by Gianfranco IZ8EWD
 * www.pianetaradio.it
 * 
 * Uses SPI interface for serial communication with the DDS and Arduino terminal for command
 * 
 * Hardware connections (Arduino UNO/Nano): 
 * AD9851
 * FQ_UD   <- D3
 * W_CLK   <- D13
 * DATA/D7 <- D11
 * 
 * LCD
 * RS     <- D8
 * RW     -> GND
 * E      <- D9
 * D4     <- D4
 * D5     <- D5
 * D6     <- D6
 * D7     <- D7
 * LED    <- D10
 * 
 * Buttons
 * btn    -> A0
 * 
 * 
 * v 1.0 14/11/2019
 *    first issue
 * 
 * v 1.1 16/11/2019
 *    + LCD display with frequency
 *    
 * v 1.2 23/11/2019
 *    + Analog keyboard
 *    * New pin configuration for compatibility with LCD Keyboard shield
 *    
 * v 1.3 28/11/2019
 *    * Solved bug that sometimes prevent DDS initialization
 *    + Added "init" command
 */

//Libraries
#include "CommandLine.h" //Command line parser
#include "DDS.h"  //DDS AD9851 control
#include "Display.h"  //LCD display
#include "Keyboard.h" //Keyboard


//Constants


//Variables
bool btnDone = false;
bool modeSelect = false;
int lcdCursor = 0;
unsigned long stepFreq = 10000000;


void setup() {

//Port definition
  pinMode(FQ_UD, OUTPUT);
  pinMode(W_CLK, OUTPUT);
  pinMode(DATA, OUTPUT);
  
  pinMode(LED, OUTPUT);
  digitalWrite(LED, HIGH);

//Serial port
  Serial.begin(9600);
  Serial.println("DDS Signal Generator");
  Serial.println("type help for info");

  ddsFreq = 10000000;   //default start frequency 10MHZ

//LCD
  displayInit();
  displayFrequency(ddsFreq);

//DDS
  ddsInit();
  ddsSendFrequency(ddsFreq);

}


void loop() {
  if (btnReleased()) btnDone = false;
  if (btnHold()) btnDone = false;
  if (btnPressed() && (btnDone == false)) {
    switch (btnRead())
    {
      case btnRIGHT:
      {
        if ((lcdCursor < 9) && (modeSelect)) {
          lcdCursor = lcdCursor + 1;
          if ((lcdCursor == 2) || (lcdCursor == 6)) lcdCursor = lcdCursor + 1;
          stepFreq = stepFreq / 10;
          lcd.setCursor(lcdCursor,0);
        }
        btnDone = true;
        break;
      }
      case btnLEFT:
      {
        if ((lcdCursor > 0) && (modeSelect)) {
          lcdCursor = lcdCursor - 1;
          if ((lcdCursor == 2) || (lcdCursor == 6)) lcdCursor = lcdCursor - 1;
          stepFreq = stepFreq * 10;
          lcd.setCursor(lcdCursor,0);
        }
        btnDone = true;
        break;
      }
      case btnUP:
      {
        if (modeSelect) {
          lcdFreq = lcdFreq + stepFreq;
          displayFrequency(lcdFreq);
          lcd.setCursor(lcdCursor,0);
        }
        btnDone = true;
        break;
      }
      case btnDOWN:
      {
        if (modeSelect) {
          lcdFreq = lcdFreq - stepFreq;
          displayFrequency(lcdFreq);
          lcd.setCursor(lcdCursor,0);
        }
        btnDone = true;
        break;
      }
      case btnSELECT:
      {
        lcd.setCursor(lcdCursor,0);
        lcd.cursor();
        btnDone = true;

        if (modeSelect) {
          lcd.noCursor();
          modeSelect = false;
          ddsSendFrequency(lcdFreq);
          displayFrequency(ddsFreq);
        }
        else modeSelect = true;
        
        break;
      }
      case btnNONE:
      {
        break;
      }
    }
  }

  
  bool clReceived = clGetCommand(CommandLine);
  
  if (clReceived) {
    clDoCommand(CommandLine);
    switch (clCommand) {
      case 0: //unknown command
        Serial.println("?");
        break;
      case 1: //freq
        ddsSendFrequency(clResult);
        displayFrequency(ddsFreq);
       break;
      case 2: //help
        Serial.println("DDS Signal Generator, by iz8ewd (www.pianetaradio.it)");
        Serial.println("v. 1.3");
        Serial.println("Help:");
        Serial.println("freq - set the DDS frequency (Hz)");
        Serial.println("cal - frequency calibration");
        Serial.println("down - DDS shut down");
        Serial.println("init - DDS init");
        break;
      case 3: //cal
        ddsCalibrate (clResult);
        break;
      case 4: //down
        ddsDown ();
        break;
      case 5: //init
        ddsInit();
        ddsFreq = 10000000;
        ddsSendFrequency(ddsFreq);
        displayFrequency(ddsFreq);
        break;        
      default:
        Serial.println("?");
    }
  }
}
