/* Command Line Parser
 * derived from "Simple Command Line Interface"
 * https://create.arduino.cc/projecthub/mikefarr/simple-command-line-interface-4f0a3f
 */

#include <string.h>
#include <stdlib.h>

#define CR '\r'
#define LF '\n'
#define BS '\b'
#define NULLCHAR '\0'
#define SPACE ' '

#define COMMAND_BUFFER_LENGTH 25  //length of serial buffer for incoming commands, max 255 chars

char CommandLine[COMMAND_BUFFER_LENGTH + 1];  //Read commands into this buffer from Serial.  +1 in length for a termination char
int clCommand;
long clResult;

const char *delimiters = ", \n";                    //commands can be separated by return, space or comma

//List of defined command
const char *helpCommand = "help";
const char *freqCommand = "freq";
const char *calCommand = "cal";
const char *downCommand = "down";
const char *initCommand = "init";


//Read command from command line
bool clGetCommand(char * commandLine) {
//parse the command
  static uint8_t charsRead = 0;
  
  while (Serial.available()) {  //read asynchronously until full command input
    char c = Serial.read();
    switch (c) {
      case CR:      //likely have full command in buffer now, commands are terminated by CR and/or LS
      case LF:
        commandLine[charsRead] = NULLCHAR;       //null terminate our command char array
        if (charsRead > 0)  {
          charsRead = 0;                           //charsRead is static, so have to reset
          //Serial.println(commandLine);
          return true;
        }
        break;
      case BS:                                    // handle backspace in input: put a space in last char
        if (charsRead > 0) {                        //and adjust commandLine and charsRead
          commandLine[--charsRead] = NULLCHAR;
          Serial << byte(BS) << byte(SPACE) << byte(BS);
        }
        break;
      default:
        // c = tolower(c);
        if (charsRead < COMMAND_BUFFER_LENGTH) {
          commandLine[charsRead++] = c;
        }
        commandLine[charsRead] = NULLCHAR;
        break;
    }
  }
  return false;
}


//Return a long signed integer from the command line
long ReadNumber () {
  char * numTextPtr = strtok(NULL, delimiters);
  return atol(numTextPtr);
}


//Execute command
void clDoCommand(char * commandLine) {

  char * ptrToCommandName = strtok(commandLine, delimiters);

  if (strcmp(ptrToCommandName, helpCommand) == 0) {
      clCommand = 2;
      clResult = 0;

  } else {
    if (strcmp(ptrToCommandName, freqCommand) == 0) {
      clCommand = 1;
      clResult = ReadNumber();

  } else {
    if (strcmp(ptrToCommandName, calCommand) == 0) {
      clCommand = 3;
      clResult = ReadNumber();

  } else {
    if (strcmp(ptrToCommandName, downCommand) == 0) {
      clCommand = 4;

  } else {
    if (strcmp(ptrToCommandName, initCommand) == 0) {
      clCommand = 5;
   
  } else {
      clCommand = 0;
      clResult = 0;
    }
  }
  }
  }
  }
}
