/* AD9851 command
 * derived from "Arduino library for AD9850 and AD9851"
 * https://github.com/k2zia/AD985XSPI
*/

#include <SPI.h>  //https://www.arduino.cc/en/reference/SPI

#define FQ_UD 3   //DDS 8 Frequency Update
#define W_CLK 13  //DDS 7 Word Load Clock, Arduino Uno/Nano SPI SCK pin
#define DATA 11   //DDS 25 Data Input D7, Arduino Uno/Nano SPI MOSI pin

#define SPIRate 8000000 //SPI baud rate

#define xREFCLK 1 //DDS enable 6x internal clock multiplier 0 = off ; 1 = on

unsigned long sysClock = 180000000;
unsigned long ddsFreq;


// Pulse high the selected pin
void pulseHigh(int pin) {
  digitalWrite(pin, HIGH);
  digitalWrite(pin, LOW);
}


//set serial load enable (D0 and D1 hardwired to high, D2 to ground) 
void ddsSerial() {
  pulseHigh(W_CLK);
  pulseHigh(FQ_UD);
}


//DDS and SPI init
void ddsInit() {
  SPI.begin();
  ddsSerial();
}


//Set DDS frequency
void ddsSendFrequency(unsigned long frequency) {
  ddsFreq = frequency;
  int32_t freq = frequency * 4294967296 / sysClock;   //fout = sys clock * frequency tuning word / 2^32
   
  SPI.beginTransaction(SPISettings(SPIRate, LSBFIRST, SPI_MODE0));  //open SPI, LSB first, CPOL 0, CPHA 0
  
  for (int i=0; i<4; i++, freq>>=8) {   //send each byte of the frequency, lowest byte first
    SPI.transfer(freq & 0xFF);
    //Serial.print(freq & 0xFF);
  }

// control byte
#ifdef xREFCLK
  SPI.transfer(0x001);
  //Serial.print(1);
#else
  SPI.transfer(0x000);
  //Serial.print(0);
#endif //xREFCLK
    
  pulseHigh(FQ_UD);

  SPI.endTransaction();
}


//calibration routine
void ddsCalibrate(double realFrequency) {
  sysClock = realFrequency * sysClock / ddsFreq;
  Serial.println(sysClock);
  ddsSendFrequency(ddsFreq);
}


//DDS shut down
void ddsDown() {
  pulseHigh(FQ_UD);
  SPI.beginTransaction(SPISettings(SPIRate, LSBFIRST, SPI_MODE0));
  SPI.transfer(0x04);
  pulseHigh(FQ_UD);
  SPI.endTransaction();
  //Serial.println("Down");
}
